@@ -73,9 +73,8 @@ static void acc_mgr_enable_rotation_cond(struct acc_mgr *acc_mgr)
 		if (!osmo_timer_pending(&acc_mgr->rotate_timer))
 			osmo_timer_schedule(&acc_mgr->rotate_timer, acc_mgr->rotation_time_sec, 0);
 	} else {
-		/* No rotation needed, disable rotation timer */
-		if (osmo_timer_pending(&acc_mgr->rotate_timer))
-			osmo_timer_del(&acc_mgr->rotate_timer);
+		/* No rotation needed, disable rotation timer (if pending) */
+		osmo_timer_del(&acc_mgr->rotate_timer);
 	}
 }
 
@@ -297,6 +296,11 @@ void acc_mgr_init(struct acc_mgr *acc_mgr, struct gsm_bts *bts)
 	acc_mgr_gen_subset(acc_mgr, false);
 }
 
+void acc_mgr_deinit(struct acc_mgr *acc_mgr)
+{
+	osmo_timer_del(&acc_mgr->rotate_timer);
+}
+
 uint8_t acc_mgr_get_len_allowed_adm(struct acc_mgr *acc_mgr)
 {
 	return acc_mgr->len_allowed_adm;
@@ -475,6 +479,17 @@ void acc_ramp_init(struct acc_ramp *acc_ramp, struct gsm_bts *bts)
 	osmo_timer_setup(&acc_ramp->step_timer, do_acc_ramping_step, acc_ramp);
 }
 
+/*!
+ * Deinitialize an acc_ramp data structure.
+ * Called when the user is going to free the structure.
+ *
+ * \param[in] acc_ramp Pointer to acc_ramp structure to be deinitialized.
+ */
+void acc_ramp_deinit(struct acc_ramp *acc_ramp)
+{
+	osmo_timer_del(&acc_ramp->step_timer);
+}
+
 /*!
  * Change the ramping step size which controls how many ACCs will be allowed per ramping step.
  * Returns negative on error (step_size out of range), else zero.
@@ -565,8 +580,7 @@ void acc_ramp_trigger(struct acc_ramp *acc_ramp)
  */
 void acc_ramp_abort(struct acc_ramp *acc_ramp)
 {
-	if (osmo_timer_pending(&acc_ramp->step_timer))
-		osmo_timer_del(&acc_ramp->step_timer);
+	osmo_timer_del(&acc_ramp->step_timer);
 
 	acc_mgr_set_len_allowed_ramp(&acc_ramp->bts->acc_mgr, 10);
 }
