@@ -21,6 +21,8 @@
  */
 
 #include <osmocom/core/fsm.h>
+#include <osmocom/gsm/protocol/gsm_48_103.h>
+#include <osmocom/gsm/rtp_extensions.h>
 #include <osmocom/netif/rtp.h>
 #include <osmocom/mgcp_client/mgcp_client_endpoint_fsm.h>
 
@@ -70,7 +72,7 @@ struct osmo_tdef_state_timeout lchan_rtp_fsm_timeouts[32] = {
 	} while (0)
 
 /* Called from lchan_fsm_init(), does not need to be visible in lchan_rtp_fsm.h */
-void lchan_rtp_fsm_init(void)
+static __attribute__((constructor)) void lchan_rtp_fsm_init(void)
 {
 	OSMO_ASSERT(osmo_fsm_register(&lchan_rtp_fsm) == 0);
 }
@@ -143,7 +145,7 @@ static void lchan_rtp_fsm_wait_mgw_endpoint_available_onenter(struct osmo_fsm_in
 	struct osmo_mgcpc_ep_ci *use_mgwep_ci = lchan_use_mgw_endpoint_ci_bts(lchan);
 	struct mgcp_conn_peer crcx_info;
 
-	if (!is_ipaccess_bts(lchan->ts->trx->bts)) {
+	if (!is_ipa_abisip_bts(lchan->ts->trx->bts)) {
 		LOG_LCHAN_RTP(lchan, LOGL_DEBUG, "Audio link to-BTS via E1, skipping IPACC\n");
 		lchan_rtp_fsm_state_chg(LCHAN_RTP_ST_WAIT_LCHAN_READY);
 		return;
@@ -169,7 +171,7 @@ static void lchan_rtp_fsm_wait_mgw_endpoint_available_onenter(struct osmo_fsm_in
 		.x_osmo_osmux_cid = -1, /* -1 is wildcard, .x_osmo_osmux_use set below */
 	};
 	if (lchan->conn) {
-		crcx_info.call_id = lchan->conn->sccp.conn_id;
+		crcx_info.call_id = lchan->conn->sccp.conn.conn_id;
 		if (lchan->conn->sccp.msc)
 			crcx_info.x_osmo_ign = lchan->conn->sccp.msc->x_osmo_ign;
 	}
@@ -290,7 +292,7 @@ static void lchan_rtp_fsm_post_lchan_ready(struct osmo_fsm_inst *fi)
 {
 	struct gsm_lchan *lchan = lchan_rtp_fi_lchan(fi);
 
-	if (is_ipaccess_bts(lchan->ts->trx->bts))
+	if (is_ipa_abisip_bts(lchan->ts->trx->bts))
 		lchan_rtp_fsm_state_chg(LCHAN_RTP_ST_WAIT_IPACC_CRCX_ACK);
 	else
 		lchan_rtp_fsm_state_chg(LCHAN_RTP_ST_WAIT_MGW_ENDPOINT_CONFIGURED);
@@ -307,14 +309,35 @@ static void lchan_rtp_fsm_wait_ipacc_crcx_ack_onenter(struct osmo_fsm_inst *fi,
 		return;
 	}
 
-	val = ipacc_speech_mode(lchan->activate.ch_mode_rate.chan_mode, lchan->type);
-	if (val < 0) {
-		lchan_rtp_fail("Cannot determine Abis/IP speech mode for tch_mode=%s type=%s",
-			   get_value_string(gsm48_chan_mode_names, lchan->activate.ch_mode_rate.chan_mode),
-			   gsm_chan_t_name(lchan->type));
-		return;
+	if (lchan->current_ch_indctr == GSM0808_CHAN_DATA) {
+		enum rsl_ipac_rtp_csd_format_d format_d = RSL_IPAC_RTP_CSD_TRAU_BTS;
+
+		if (lchan->activate.ch_mode_rate.data_transparent) {
+			val = ipacc_rtp_csd_fmt_transp(&lchan->activate.ch_mode_rate, format_d);
+			if (val < 0) {
+				lchan_rtp_fail("Cannot determine Abis/IP RTP CSD format for rsl_cmod_csd_t=%d",
+					       lchan->activate.ch_mode_rate.data_rate.t);
+				return;
+			}
+		} else {
+			val = ipacc_rtp_csd_fmt_non_transp(&lchan->activate.ch_mode_rate, format_d);
+			if (val < 0) {
+				lchan_rtp_fail("Cannot determine Abis/IP RTP CSD format for rsl_cmod_csd_nt=%d",
+					       lchan->activate.ch_mode_rate.data_rate.nt);
+				return;
+			}
+		}
+		lchan->abis_ip.rtp_csd_fmt = val;
+	} else {
+		val = ipacc_speech_mode(lchan->activate.ch_mode_rate.chan_mode, lchan->type);
+		if (val < 0) {
+			lchan_rtp_fail("Cannot determine Abis/IP speech mode for tch_mode=%s type=%s",
+				   get_value_string(gsm48_chan_mode_names, lchan->activate.ch_mode_rate.chan_mode),
+				   gsm_chan_t_name(lchan->type));
+			return;
+		}
+		lchan->abis_ip.speech_mode = val;
 	}
-	lchan->abis_ip.speech_mode = val;
 
 	val = ipacc_payload_type(lchan->activate.ch_mode_rate.chan_mode, lchan->type);
 	if (val < 0) {
@@ -495,7 +518,7 @@ static void lchan_rtp_fsm_wait_mgw_endpoint_configured_onenter(struct osmo_fsm_i
 		return;
 	}
 
-	if (!is_ipaccess_bts(lchan->ts->trx->bts)) {
+	if (!is_ipa_abisip_bts(lchan->ts->trx->bts)) {
 		LOG_LCHAN_RTP(lchan, LOGL_DEBUG, "Audio link to-BTS via E1, skipping IPACC\n");
 		lchan_rtp_fsm_state_chg(LCHAN_RTP_ST_READY);
 		return;
@@ -522,7 +545,7 @@ static void lchan_rtp_fsm_wait_mgw_endpoint_configured(struct osmo_fsm_inst *fi,
 
 	switch (event) {
 	case LCHAN_RTP_EV_MGW_ENDPOINT_CONFIGURED:
-		if (is_ipaccess_bts(lchan->ts->trx->bts))
+		if (is_ipa_abisip_bts(lchan->ts->trx->bts))
 			lchan_rtp_fsm_state_chg(LCHAN_RTP_ST_WAIT_IPACC_MDCX_ACK);
 		else {
 			lchan_rtp_fsm_state_chg(LCHAN_RTP_ST_READY);
@@ -589,7 +612,7 @@ static void lchan_rtp_fsm_rollback_onenter(struct osmo_fsm_inst *fi, uint32_t pr
 		return;
 	}
 
-	if (is_ipaccess_bts(lchan->ts->trx->bts))
+	if (is_ipa_abisip_bts(lchan->ts->trx->bts))
 		connect_mgw_endpoint_to_lchan(fi, lchan->mgw_endpoint_ci_bts, old_lchan);
 	else
 		osmo_fsm_inst_dispatch(fi, LCHAN_RTP_EV_MGW_ENDPOINT_CONFIGURED, 0);
@@ -850,6 +873,12 @@ static struct osmo_fsm lchan_rtp_fsm = {
 static enum mgcp_codecs chan_mode_to_mgcp_codec(enum gsm48_chan_mode chan_mode, bool full_rate)
 {
 	switch (gsm48_chan_mode_to_non_vamos(chan_mode)) {
+	case GSM48_CMODE_DATA_14k5:
+	case GSM48_CMODE_DATA_12k0:
+	case GSM48_CMODE_DATA_6k0:
+	case GSM48_CMODE_DATA_3k6:
+		return CODEC_CLEARMODE;
+
 	case GSM48_CMODE_SPEECH_V1:
 		if (full_rate)
 			return CODEC_GSM_8000_1;
@@ -866,9 +895,36 @@ static enum mgcp_codecs chan_mode_to_mgcp_codec(enum gsm48_chan_mode chan_mode,
 	}
 }
 
+static int chan_mode_to_mgcp_aoip_pt(enum mgcp_codecs codec)
+{
+	switch (codec) {
+	case CODEC_GSM_8000_1:
+		return OSMO_AOIP_RTP_PT_FR1;
+
+	case CODEC_GSMEFR_8000_1:
+		return OSMO_AOIP_RTP_PT_EFR;
+
+	case CODEC_GSMHR_8000_1:
+		return OSMO_AOIP_RTP_PT_HR1;
+
+	case CODEC_AMR_8000_1:
+		return OSMO_AOIP_RTP_PT_AMR;
+
+	case CODEC_CLEARMODE:
+		return OSMO_AOIP_RTP_PT_CSD;
+
+	default:
+		/* Error: unknown codec */
+		return -1;
+	}
+}
+
 static int chan_mode_to_mgcp_bss_pt(enum mgcp_codecs codec)
 {
 	switch (codec) {
+	case CODEC_GSM_8000_1:
+		return RTP_PT_GSM_FULL;
+
 	case CODEC_GSMHR_8000_1:
 		return RTP_PT_GSM_HALF;
 
@@ -878,9 +934,11 @@ static int chan_mode_to_mgcp_bss_pt(enum mgcp_codecs codec)
 	case CODEC_AMR_8000_1:
 		return RTP_PT_AMR;
 
+	case CODEC_CLEARMODE:
+		return RTP_PT_CSDATA;
+
 	default:
-		/* Not an error, we just leave it to libosmo-mgcp-client to
-		 * decide over the PT. */
+		/* Error: unknown codec */
 		return -1;
 	}
 }
@@ -889,47 +947,81 @@ void mgcp_pick_codec(struct mgcp_conn_peer *verb_info, const struct gsm_lchan *l
 {
 	enum mgcp_codecs codec = chan_mode_to_mgcp_codec(lchan->activate.ch_mode_rate.chan_mode,
 							 lchan->type == GSM_LCHAN_TCH_H? false : true);
-	int custom_pt;
+	int pt, amr_oa;
 
 	if (codec < 0) {
 		LOG_LCHAN(lchan, LOGL_ERROR,
 			  "Unable to determine MGCP codec type for %s in chan-mode %s\n",
 			  gsm_chan_t_name(lchan->type), gsm48_chan_mode_name(lchan->activate.ch_mode_rate.chan_mode));
-		verb_info->codecs_len = 0;
+		verb_info->ptmap_len = 0;
 		return;
 	}
 
-	verb_info->codecs[0] = codec;
-	verb_info->codecs_len = 1;
-
-	/* Setup custom payload types (only for BSS side and when required) */
-	custom_pt = chan_mode_to_mgcp_bss_pt(codec);
-	if (bss_side && custom_pt > 0) {
-		verb_info->ptmap[0].codec = codec;
-		verb_info->ptmap[0].pt = custom_pt;
-		verb_info->ptmap_len = 1;
+	/* The new libosmo-mgcp-client API requires us to provide explicit
+	 * payload type number for every codec - no more internal defaulting.
+	 * Legacy payload types used on IPA/Osmocom Abis-IP are defined in
+	 * <osmocom/netif/rtp.h> as RTP_PT_*, new TS 48.103 (AoIP user plane)
+	 * payload types are defined in <osmocom/gsm/protocol/gsm_48_103.h>
+	 * as OSMO_AOIP_RTP_PT_*.
+	 */
+	if (bss_side)
+		pt = chan_mode_to_mgcp_bss_pt(codec);
+	else
+		pt = chan_mode_to_mgcp_aoip_pt(codec);
+	if (pt < 0) {
+		LOG_LCHAN(lchan, LOGL_ERROR,
+			  "Unable to determine RTP payload type for %s in chan-mode %s\n",
+			  gsm_chan_t_name(lchan->type), gsm48_chan_mode_name(lchan->activate.ch_mode_rate.chan_mode));
+		verb_info->ptmap_len = 0;
+		return;
 	}
 
-	/* AMR requires additional parameters to be set up (framing mode) */
-	if (verb_info->codecs[0] == CODEC_AMR_8000_1) {
-		verb_info->param_present = true;
-		verb_info->param.amr_octet_aligned_present = true;
-	}
+	verb_info->ptmap[0].codec = codec;
+	verb_info->ptmap[0].pt = pt;
+	verb_info->ptmap_len = 1;
 
-	if (bss_side && verb_info->codecs[0] == CODEC_AMR_8000_1) {
-		/* FIXME: At the moment all BTSs we support are using the
-		 * octet-aligned payload format. However, in the future
-		 * we may support BTSs that are using bandwidth-efficient
-		 * format. In this case we will have to add functionality
-		 * that distinguishes by the BTS model which mode to use. */
-		verb_info->param.amr_octet_aligned = true;
-	}
-	else if (!bss_side && verb_info->codecs[0] == CODEC_AMR_8000_1) {
-		verb_info->param.amr_octet_aligned = lchan->conn->sccp.msc->amr_octet_aligned;
+	/* Some codecs require fmtp parameters, either always (AMR)
+	 * or conditionally (FR & EFR with TW-TS-001, HR with TW-TS-002).
+	 * Use the new fmtp string API to insert them into our outgoing
+	 * CRCX/MDCX.
+	 */
+	switch (codec) {
+	case CODEC_AMR_8000_1:
+		/* Right now AMR with TW-TS-006 is supported only on E1 BTS,
+		 * in which case there is no BSS side, only AoIP side.
+		 * However, if TW-TS-006 support gets added to IP-native
+		 * OsmoBTS in the future, it will strictly require no-alteration
+		 * pass-through from the BSC-associated MGW.  Therefore,
+		 * indicate this format on both sides when it is enabled. */
+		if (lchan->conn->user_plane.rtp_extensions & OSMO_RTP_EXT_TWTS006) {
+			OSMO_STRLCPY_ARRAY(verb_info->ptmap[0].fmtp,
+					   "octet-align=1;tw-ts-006=1");
+			break;
+		}
+		/* standard 3GPP/Osmocom/etc operation with RFC 4867 format */
+		if (bss_side)
+			amr_oa = 1;
+		else
+			amr_oa = lchan->conn->sccp.msc->amr_octet_aligned;
+		snprintf(verb_info->ptmap[0].fmtp,
+			 sizeof(verb_info->ptmap[0].fmtp),
+			 "octet-align=%d", amr_oa);
+		break;
+	case CODEC_GSM_8000_1:
+	case CODEC_GSMEFR_8000_1:
+		if (lchan->conn->user_plane.rtp_extensions & OSMO_RTP_EXT_TWTS001)
+			OSMO_STRLCPY_ARRAY(verb_info->ptmap[0].fmtp, "tw-ts-001=1");
+		break;
+	case CODEC_GSMHR_8000_1:
+		if (lchan->conn->user_plane.rtp_extensions & OSMO_RTP_EXT_TWTS002)
+			OSMO_STRLCPY_ARRAY(verb_info->ptmap[0].fmtp, "tw-ts-002=1");
+		break;
+	default:
+		break;
 	}
 }
 
 bool mgcp_codec_is_picked(const struct mgcp_conn_peer *verb_info, enum mgcp_codecs codec)
 {
-	return verb_info->codecs[0] == codec;
+	return verb_info->ptmap[0].codec == codec;
 }
